/*----- PROTECTED REGION ID(HipaceClass.cpp) ENABLED START -----*/
static const char *RcsId      = "$Id: HipaceClass.cpp,v 1.7 2020-02-18 17:20:49 alessio Exp $";
static const char *TagName    = "Hipace $Name:  $";
static const char *CvsPath    = "$Source: /home/cvsadm/cvsroot/fermi/servers/hipace/src/HipaceClass.cpp,v $";
static const char *SvnPath    = "$HeadURL:  $";
static const char *HttpServer = "http://www.esrf.eu/computing/cs/tango/tango_doc/ds_doc/";
//=============================================================================
//
// file :        HipaceClass.cpp
//
// description : C++ source for the HipaceClass.
//               A singleton class derived from DeviceClass.
//               It implements the command and attribute list
//               and all properties and methods required
//               by the Hipace once per process.
//
// project :     
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
// $Author: alessio $
//
// $Revision: 1.7 $
// $Date: 2020-02-18 17:20:49 $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <HipaceClass.h>

/*----- PROTECTED REGION END -----*/	//	HipaceClass.cpp

//-------------------------------------------------------------------
/**
 *	Create HipaceClass singleton and
 *	return it in a C function for Python usage
 */
//-------------------------------------------------------------------
extern "C" {
#ifdef _TG_WINDOWS_

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_Hipace_class(const char *name) {
		return Hipace_ns::HipaceClass::init(name);
	}
}

namespace Hipace_ns
{
//===================================================================
//	Initialize pointer for singleton pattern
//===================================================================
HipaceClass *HipaceClass::_instance = NULL;

//--------------------------------------------------------
/**
 * method : 		HipaceClass::HipaceClass(string &s)
 * description : 	constructor for the HipaceClass
 *
 * @param s	The class name
 */
//--------------------------------------------------------
HipaceClass::HipaceClass(string &s):Serial2Client_ns::Serial2ClientClass(s)
{
	cout2 << "Entering HipaceClass constructor" << endl;
	set_default_property();
	write_class_property();

	/*----- PROTECTED REGION ID(HipaceClass::constructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::constructor

	cout2 << "Leaving HipaceClass constructor" << endl;
}

//--------------------------------------------------------
/**
 * method : 		HipaceClass::~HipaceClass()
 * description : 	destructor for the HipaceClass
 */
//--------------------------------------------------------
HipaceClass::~HipaceClass()
{
	/*----- PROTECTED REGION ID(HipaceClass::destructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::destructor

	_instance = NULL;
}


//--------------------------------------------------------
/**
 * method : 		HipaceClass::init
 * description : 	Create the object if not already done.
 *                  Otherwise, just return a pointer to the object
 *
 * @param	name	The class name
 */
//--------------------------------------------------------
HipaceClass *HipaceClass::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			string s(name);
			_instance = new HipaceClass(s);
		}
		catch (bad_alloc &)
		{
			throw;
		}
	}
	return _instance;
}

//--------------------------------------------------------
/**
 * method : 		HipaceClass::instance
 * description : 	Check if object already created,
 *                  and return a pointer to the object
 */
//--------------------------------------------------------
HipaceClass *HipaceClass::instance()
{
	if (_instance == NULL)
	{
		cerr << "Class is not initialised !!" << endl;
		exit(-1);
	}
	return _instance;
}



//===================================================================
//	Command execution method calls
//===================================================================

//===================================================================
//	Properties management
//===================================================================
//--------------------------------------------------------
/**
 *	Method      : HipaceClass::get_class_property()
 *	Description : Get the class property for specified name.
 */
//--------------------------------------------------------
Tango::DbDatum HipaceClass::get_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, returns  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : HipaceClass::get_default_device_property()
 *	Description : Return the default value for device property.
 */
//--------------------------------------------------------
Tango::DbDatum HipaceClass::get_default_device_property(string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : HipaceClass::get_default_class_property()
 *	Description : Return the default value for class property.
 */
//--------------------------------------------------------
Tango::DbDatum HipaceClass::get_default_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}


//--------------------------------------------------------
/**
 *	Method      : HipaceClass::set_default_property()
 *	Description : Set default property (class and device) for wizard.
 *                For each property, add to wizard property name and description.
 *                If default value has been set, add it to wizard property and
 *                store it in a DbDatum.
 */
//--------------------------------------------------------
void HipaceClass::set_default_property()
{
	string	prop_name;
	string	prop_desc;
	string	prop_def;
	vector<string>	vect_data;

	//	Set Default Class Properties

	//	Set Default device Properties
	prop_name = "DeviceAddress";
	prop_desc = "Rs485`s address";
	prop_def  = "001";
	vect_data.clear();
	vect_data.push_back("001");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
}

//--------------------------------------------------------
/**
 *	Method      : HipaceClass::write_class_property()
 *	Description : Set class description fields as property in database
 */
//--------------------------------------------------------
void HipaceClass::write_class_property()
{
	//	First time, check if database used
	if (Tango::Util::_UseDb == false)
		return;

	Tango::DbData	data;
	string	classname = get_name();
	string	header;
	string::size_type	start, end;

	//	Put title
	Tango::DbDatum	title("ProjectTitle");
	string	str_title("");
	title << str_title;
	data.push_back(title);

	//	Put Description
	Tango::DbDatum	description("Description");
	vector<string>	str_desc;
	str_desc.push_back("");
	description << str_desc;
	data.push_back(description);

	//	put cvs or svn location
	string	filename("Hipace");
	filename += "Class.cpp";

	// check for cvs information
	string	src_path(CvsPath);
	start = src_path.find("/");
	if (start!=string::npos)
	{
		end   = src_path.find(filename);
		if (end>start)
		{
			string	strloc = src_path.substr(start, end-start);
			//	Check if specific repository
			start = strloc.find("/cvsroot/");
			if (start!=string::npos && start>0)
			{
				string	repository = strloc.substr(0, start);
				if (repository.find("/segfs/")!=string::npos)
					strloc = "ESRF:" + strloc.substr(start, strloc.length()-start);
			}
			Tango::DbDatum	cvs_loc("cvs_location");
			cvs_loc << strloc;
			data.push_back(cvs_loc);
		}
	}

	// check for svn information
	else
	{
		string	src_path(SvnPath);
		start = src_path.find("://");
		if (start!=string::npos)
		{
			end = src_path.find(filename);
			if (end>start)
			{
				header = "$HeadURL: ";
				start = header.length();
				string	strloc = src_path.substr(start, (end-start));
				
				Tango::DbDatum	svn_loc("svn_location");
				svn_loc << strloc;
				data.push_back(svn_loc);
			}
		}
	}

	//	Get CVS or SVN revision tag
	
	// CVS tag
	string	tagname(TagName);
	header = "$Name: ";
	start = header.length();
	string	endstr(" $");
	
	end   = tagname.find(endstr);
	if (end!=string::npos && end>start)
	{
		string	strtag = tagname.substr(start, end-start);
		Tango::DbDatum	cvs_tag("cvs_tag");
		cvs_tag << strtag;
		data.push_back(cvs_tag);
	}
	
	// SVN tag
	string	svnpath(SvnPath);
	header = "$HeadURL: ";
	start = header.length();
	
	end   = svnpath.find(endstr);
	if (end!=string::npos && end>start)
	{
		string	strloc = svnpath.substr(start, end-start);
		
		string tagstr ("/tags/");
		start = strloc.find(tagstr);
		if ( start!=string::npos )
		{
			start = start + tagstr.length();
			end   = strloc.find(filename);
			string	strtag = strloc.substr(start, end-start-1);
			
			Tango::DbDatum	svn_tag("svn_tag");
			svn_tag << strtag;
			data.push_back(svn_tag);
		}
	}

	//	Get URL location
	string	httpServ(HttpServer);
	if (httpServ.length()>0)
	{
		Tango::DbDatum	db_doc_url("doc_url");
		db_doc_url << httpServ;
		data.push_back(db_doc_url);
	}

	//  Put inheritance
	Tango::DbDatum	inher_datum("InheritedFrom");
	vector<string> inheritance;
	inheritance.push_back("TANGO_BASE_CLASS");
	inher_datum << inheritance;
	data.push_back(inher_datum);

	//	Call database and and values
	get_db_class()->put_property(data);
}

//===================================================================
//	Factory methods
//===================================================================

//--------------------------------------------------------
/**
 *	Method      : HipaceClass::device_factory()
 *	Description : Create the device object(s)
 *                and store them in the device list
 */
//--------------------------------------------------------
void HipaceClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{
	/*----- PROTECTED REGION ID(HipaceClass::device_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::device_factory_before

	//	Create devices and add it into the device list
	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		cout4 << "Device name : " << (*devlist_ptr)[i].in() << endl;
		device_list.push_back(new Hipace(this, (*devlist_ptr)[i]));
	}

	//	Manage dynamic attributes if any
	erase_dynamic_attributes(devlist_ptr, get_class_attr()->get_attr_list());

	//	Export devices to the outside world
	for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++)
	{
		//	Add dynamic attributes if any
		Hipace *dev = static_cast<Hipace *>(device_list[device_list.size()-i]);
		dev->add_dynamic_attributes();

		//	Check before if database used.
		if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
			export_device(dev);
		else
			export_device(dev, dev->get_name().c_str());
	}

	/*----- PROTECTED REGION ID(HipaceClass::device_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::device_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : HipaceClass::attribute_factory()
 *	Description : Create the attribute object(s)
 *                and store them in the attribute list
 */
//--------------------------------------------------------
void HipaceClass::attribute_factory(vector<Tango::Attr *> &att_list)
{
	/*----- PROTECTED REGION ID(HipaceClass::attribute_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::attribute_factory_before
	//	Call atribute_factory for inherited class
	Serial2Client_ns::Serial2ClientClass::attribute_factory(att_list);

	//	Attribute : RemotePrio
	RemotePrioAttrib	*remoteprio = new RemotePrioAttrib();
	Tango::UserDefaultAttrProp	remoteprio_prop;
	remoteprio_prop.set_description("Priorita` Remote");
	remoteprio_prop.set_label("RemotePrio (300)");
	//	unit	not set for RemotePrio
	//	standard_unit	not set for RemotePrio
	//	display_unit	not set for RemotePrio
	//	format	not set for RemotePrio
	//	max_value	not set for RemotePrio
	//	min_value	not set for RemotePrio
	//	max_alarm	not set for RemotePrio
	//	min_alarm	not set for RemotePrio
	//	max_warning	not set for RemotePrio
	//	min_warning	not set for RemotePrio
	//	delta_t	not set for RemotePrio
	//	delta_val	not set for RemotePrio
	
	remoteprio->set_default_properties(remoteprio_prop);
	//	Not Polled
	remoteprio->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(remoteprio);

	//	Attribute : SpdSwPtAtt
	SpdSwPtAttAttrib	*spdswptatt = new SpdSwPtAttAttrib();
	Tango::UserDefaultAttrProp	spdswptatt_prop;
	spdswptatt_prop.set_description("Punto di scatto velocita` raggiunto");
	spdswptatt_prop.set_label("SpdSwPtAtt (302)");
	//	unit	not set for SpdSwPtAtt
	//	standard_unit	not set for SpdSwPtAtt
	//	display_unit	not set for SpdSwPtAtt
	//	format	not set for SpdSwPtAtt
	//	max_value	not set for SpdSwPtAtt
	//	min_value	not set for SpdSwPtAtt
	//	max_alarm	not set for SpdSwPtAtt
	//	min_alarm	not set for SpdSwPtAtt
	//	max_warning	not set for SpdSwPtAtt
	//	min_warning	not set for SpdSwPtAtt
	//	delta_t	not set for SpdSwPtAtt
	//	delta_val	not set for SpdSwPtAtt
	
	spdswptatt->set_default_properties(spdswptatt_prop);
	//	Not Polled
	spdswptatt->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(spdswptatt);

	//	Attribute : ErrorCode
	ErrorCodeAttrib	*errorcode = new ErrorCodeAttrib();
	Tango::UserDefaultAttrProp	errorcode_prop;
	errorcode_prop.set_description("Codice errore");
	errorcode_prop.set_label("ErrorCode (303)");
	//	unit	not set for ErrorCode
	//	standard_unit	not set for ErrorCode
	//	display_unit	not set for ErrorCode
	//	format	not set for ErrorCode
	//	max_value	not set for ErrorCode
	//	min_value	not set for ErrorCode
	//	max_alarm	not set for ErrorCode
	//	min_alarm	not set for ErrorCode
	//	max_warning	not set for ErrorCode
	//	min_warning	not set for ErrorCode
	//	delta_t	not set for ErrorCode
	//	delta_val	not set for ErrorCode
	
	errorcode->set_default_properties(errorcode_prop);
	//	Not Polled
	errorcode->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(errorcode);

	//	Attribute : OvTempElec
	OvTempElecAttrib	*ovtempelec = new OvTempElecAttrib();
	Tango::UserDefaultAttrProp	ovtempelec_prop;
	ovtempelec_prop.set_description("Sovratemperatura apparecchiatura elettronica di comando");
	ovtempelec_prop.set_label("OvTempElec (304)");
	//	unit	not set for OvTempElec
	//	standard_unit	not set for OvTempElec
	//	display_unit	not set for OvTempElec
	//	format	not set for OvTempElec
	//	max_value	not set for OvTempElec
	//	min_value	not set for OvTempElec
	//	max_alarm	not set for OvTempElec
	//	min_alarm	not set for OvTempElec
	//	max_warning	not set for OvTempElec
	//	min_warning	not set for OvTempElec
	//	delta_t	not set for OvTempElec
	//	delta_val	not set for OvTempElec
	
	ovtempelec->set_default_properties(ovtempelec_prop);
	//	Not Polled
	ovtempelec->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(ovtempelec);

	//	Attribute : OvTempPump
	OvTempPumpAttrib	*ovtemppump = new OvTempPumpAttrib();
	Tango::UserDefaultAttrProp	ovtemppump_prop;
	ovtemppump_prop.set_description("Sovratemperatura pompe");
	ovtemppump_prop.set_label("OvTempPump (305)");
	//	unit	not set for OvTempPump
	//	standard_unit	not set for OvTempPump
	//	display_unit	not set for OvTempPump
	//	format	not set for OvTempPump
	//	max_value	not set for OvTempPump
	//	min_value	not set for OvTempPump
	//	max_alarm	not set for OvTempPump
	//	min_alarm	not set for OvTempPump
	//	max_warning	not set for OvTempPump
	//	min_warning	not set for OvTempPump
	//	delta_t	not set for OvTempPump
	//	delta_val	not set for OvTempPump
	
	ovtemppump->set_default_properties(ovtemppump_prop);
	//	Not Polled
	ovtemppump->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(ovtemppump);

	//	Attribute : SetSpdAtt
	SetSpdAttAttrib	*setspdatt = new SetSpdAttAttrib();
	Tango::UserDefaultAttrProp	setspdatt_prop;
	setspdatt_prop.set_description("Velocita` nominale raggiunta");
	setspdatt_prop.set_label("SetSpdAtt (306)");
	//	unit	not set for SetSpdAtt
	//	standard_unit	not set for SetSpdAtt
	//	display_unit	not set for SetSpdAtt
	//	format	not set for SetSpdAtt
	//	max_value	not set for SetSpdAtt
	//	min_value	not set for SetSpdAtt
	//	max_alarm	not set for SetSpdAtt
	//	min_alarm	not set for SetSpdAtt
	//	max_warning	not set for SetSpdAtt
	//	min_warning	not set for SetSpdAtt
	//	delta_t	not set for SetSpdAtt
	//	delta_val	not set for SetSpdAtt
	
	setspdatt->set_default_properties(setspdatt_prop);
	//	Not Polled
	setspdatt->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(setspdatt);

	//	Attribute : PumpAccel
	PumpAccelAttrib	*pumpaccel = new PumpAccelAttrib();
	Tango::UserDefaultAttrProp	pumpaccel_prop;
	pumpaccel_prop.set_description("Pompa accelerata");
	pumpaccel_prop.set_label("PumpAccel (307)");
	//	unit	not set for PumpAccel
	//	standard_unit	not set for PumpAccel
	//	display_unit	not set for PumpAccel
	//	format	not set for PumpAccel
	//	max_value	not set for PumpAccel
	//	min_value	not set for PumpAccel
	//	max_alarm	not set for PumpAccel
	//	min_alarm	not set for PumpAccel
	//	max_warning	not set for PumpAccel
	//	min_warning	not set for PumpAccel
	//	delta_t	not set for PumpAccel
	//	delta_val	not set for PumpAccel
	
	pumpaccel->set_default_properties(pumpaccel_prop);
	//	Not Polled
	pumpaccel->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(pumpaccel);

	//	Attribute : DrvCurrent
	DrvCurrentAttrib	*drvcurrent = new DrvCurrentAttrib();
	Tango::UserDefaultAttrProp	drvcurrent_prop;
	drvcurrent_prop.set_description("Corrente di avviamento");
	drvcurrent_prop.set_label("DrvCurrent (310)");
	drvcurrent_prop.set_unit("A");
	//	standard_unit	not set for DrvCurrent
	//	display_unit	not set for DrvCurrent
	//	format	not set for DrvCurrent
	drvcurrent_prop.set_max_value("9999.99");
	drvcurrent_prop.set_min_value("0");
	//	max_alarm	not set for DrvCurrent
	//	min_alarm	not set for DrvCurrent
	//	max_warning	not set for DrvCurrent
	//	min_warning	not set for DrvCurrent
	//	delta_t	not set for DrvCurrent
	//	delta_val	not set for DrvCurrent
	
	drvcurrent->set_default_properties(drvcurrent_prop);
	//	Not Polled
	drvcurrent->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(drvcurrent);

	//	Attribute : OpHrsPump
	OpHrsPumpAttrib	*ophrspump = new OpHrsPumpAttrib();
	Tango::UserDefaultAttrProp	ophrspump_prop;
	ophrspump_prop.set_description("Ore operative pompa");
	ophrspump_prop.set_label("OpHrsPump (311)");
	ophrspump_prop.set_unit("h");
	//	standard_unit	not set for OpHrsPump
	//	display_unit	not set for OpHrsPump
	//	format	not set for OpHrsPump
	ophrspump_prop.set_max_value("65535");
	ophrspump_prop.set_min_value("0");
	//	max_alarm	not set for OpHrsPump
	//	min_alarm	not set for OpHrsPump
	//	max_warning	not set for OpHrsPump
	//	min_warning	not set for OpHrsPump
	//	delta_t	not set for OpHrsPump
	//	delta_val	not set for OpHrsPump
	
	ophrspump->set_default_properties(ophrspump_prop);
	//	Not Polled
	ophrspump->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(ophrspump);

	//	Attribute : FwVersion
	FwVersionAttrib	*fwversion = new FwVersionAttrib();
	Tango::UserDefaultAttrProp	fwversion_prop;
	fwversion_prop.set_description("Versione software apparecchiatura elettronica di comando");
	fwversion_prop.set_label("FwVersion (312)");
	//	unit	not set for FwVersion
	//	standard_unit	not set for FwVersion
	//	display_unit	not set for FwVersion
	//	format	not set for FwVersion
	//	max_value	not set for FwVersion
	//	min_value	not set for FwVersion
	//	max_alarm	not set for FwVersion
	//	min_alarm	not set for FwVersion
	//	max_warning	not set for FwVersion
	//	min_warning	not set for FwVersion
	//	delta_t	not set for FwVersion
	//	delta_val	not set for FwVersion
	
	fwversion->set_default_properties(fwversion_prop);
	//	Not Polled
	fwversion->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(fwversion);

	//	Attribute : DrvVoltage
	DrvVoltageAttrib	*drvvoltage = new DrvVoltageAttrib();
	Tango::UserDefaultAttrProp	drvvoltage_prop;
	drvvoltage_prop.set_description("Tensione di avviamento");
	drvvoltage_prop.set_label("DrvVoltage (313)");
	drvvoltage_prop.set_unit("V");
	//	standard_unit	not set for DrvVoltage
	//	display_unit	not set for DrvVoltage
	//	format	not set for DrvVoltage
	drvvoltage_prop.set_max_value("9999.99");
	drvvoltage_prop.set_min_value("0");
	//	max_alarm	not set for DrvVoltage
	//	min_alarm	not set for DrvVoltage
	//	max_warning	not set for DrvVoltage
	//	min_warning	not set for DrvVoltage
	//	delta_t	not set for DrvVoltage
	//	delta_val	not set for DrvVoltage
	
	drvvoltage->set_default_properties(drvvoltage_prop);
	//	Not Polled
	drvvoltage->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(drvvoltage);

	//	Attribute : OpHrsElec
	OpHrsElecAttrib	*ophrselec = new OpHrsElecAttrib();
	Tango::UserDefaultAttrProp	ophrselec_prop;
	ophrselec_prop.set_description("Ore operative apparecchiatura elettronica di comando");
	ophrselec_prop.set_label("OpHrsElec (314)");
	ophrselec_prop.set_unit("h");
	//	standard_unit	not set for OpHrsElec
	//	display_unit	not set for OpHrsElec
	//	format	not set for OpHrsElec
	ophrselec_prop.set_max_value("65535");
	ophrselec_prop.set_min_value("0");
	//	max_alarm	not set for OpHrsElec
	//	min_alarm	not set for OpHrsElec
	//	max_warning	not set for OpHrsElec
	//	min_warning	not set for OpHrsElec
	//	delta_t	not set for OpHrsElec
	//	delta_val	not set for OpHrsElec
	
	ophrselec->set_default_properties(ophrselec_prop);
	//	Not Polled
	ophrselec->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(ophrselec);

	//	Attribute : DrvPower
	DrvPowerAttrib	*drvpower = new DrvPowerAttrib();
	Tango::UserDefaultAttrProp	drvpower_prop;
	drvpower_prop.set_description("Potenza motrice");
	drvpower_prop.set_label("DrvPower (316)");
	drvpower_prop.set_unit("W");
	//	standard_unit	not set for DrvPower
	//	display_unit	not set for DrvPower
	//	format	not set for DrvPower
	drvpower_prop.set_max_value("999999");
	drvpower_prop.set_min_value("0");
	//	max_alarm	not set for DrvPower
	//	min_alarm	not set for DrvPower
	//	max_warning	not set for DrvPower
	//	min_warning	not set for DrvPower
	//	delta_t	not set for DrvPower
	//	delta_val	not set for DrvPower
	
	drvpower->set_default_properties(drvpower_prop);
	//	Not Polled
	drvpower->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(drvpower);

	//	Attribute : PumpCycles
	PumpCyclesAttrib	*pumpcycles = new PumpCyclesAttrib();
	Tango::UserDefaultAttrProp	pumpcycles_prop;
	pumpcycles_prop.set_description("Cicli pompa");
	pumpcycles_prop.set_label("PumpCycles (319)");
	//	unit	not set for PumpCycles
	//	standard_unit	not set for PumpCycles
	//	display_unit	not set for PumpCycles
	//	format	not set for PumpCycles
	pumpcycles_prop.set_max_value("65535");
	pumpcycles_prop.set_min_value("0");
	//	max_alarm	not set for PumpCycles
	//	min_alarm	not set for PumpCycles
	//	max_warning	not set for PumpCycles
	//	min_warning	not set for PumpCycles
	//	delta_t	not set for PumpCycles
	//	delta_val	not set for PumpCycles
	
	pumpcycles->set_default_properties(pumpcycles_prop);
	//	Not Polled
	pumpcycles->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(pumpcycles);

	//	Attribute : ElecName
	ElecNameAttrib	*elecname = new ElecNameAttrib();
	Tango::UserDefaultAttrProp	elecname_prop;
	elecname_prop.set_description("Denominazione apparecchiatura elettronica di comando");
	elecname_prop.set_label("ElecName (349)");
	//	unit	not set for ElecName
	//	standard_unit	not set for ElecName
	//	display_unit	not set for ElecName
	//	format	not set for ElecName
	//	max_value	not set for ElecName
	//	min_value	not set for ElecName
	//	max_alarm	not set for ElecName
	//	min_alarm	not set for ElecName
	//	max_warning	not set for ElecName
	//	min_warning	not set for ElecName
	//	delta_t	not set for ElecName
	//	delta_val	not set for ElecName
	
	elecname->set_default_properties(elecname_prop);
	//	Not Polled
	elecname->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(elecname);

	//	Attribute : SetRotSpdHz
	SetRotSpdHzAttrib	*setrotspdhz = new SetRotSpdHzAttrib();
	Tango::UserDefaultAttrProp	setrotspdhz_prop;
	setrotspdhz_prop.set_description("Velocita` nominale");
	setrotspdhz_prop.set_label("SetRotSpdHz (308)");
	setrotspdhz_prop.set_unit("Hz");
	//	standard_unit	not set for SetRotSpdHz
	//	display_unit	not set for SetRotSpdHz
	//	format	not set for SetRotSpdHz
	setrotspdhz_prop.set_max_value("999999");
	setrotspdhz_prop.set_min_value("0");
	//	max_alarm	not set for SetRotSpdHz
	//	min_alarm	not set for SetRotSpdHz
	//	max_warning	not set for SetRotSpdHz
	//	min_warning	not set for SetRotSpdHz
	//	delta_t	not set for SetRotSpdHz
	//	delta_val	not set for SetRotSpdHz
	
	setrotspdhz->set_default_properties(setrotspdhz_prop);
	//	Not Polled
	setrotspdhz->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(setrotspdhz);

	//	Attribute : ActualSpdHz
	ActualSpdHzAttrib	*actualspdhz = new ActualSpdHzAttrib();
	Tango::UserDefaultAttrProp	actualspdhz_prop;
	actualspdhz_prop.set_description("Velocita` effettiva");
	actualspdhz_prop.set_label("ActualSpdHz (309)");
	actualspdhz_prop.set_unit("Hz");
	//	standard_unit	not set for ActualSpdHz
	//	display_unit	not set for ActualSpdHz
	//	format	not set for ActualSpdHz
	actualspdhz_prop.set_max_value("999999");
	actualspdhz_prop.set_min_value("0");
	//	max_alarm	not set for ActualSpdHz
	//	min_alarm	not set for ActualSpdHz
	//	max_warning	not set for ActualSpdHz
	//	min_warning	not set for ActualSpdHz
	//	delta_t	not set for ActualSpdHz
	//	delta_val	not set for ActualSpdHz
	
	actualspdhz->set_default_properties(actualspdhz_prop);
	//	Not Polled
	actualspdhz->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(actualspdhz);

	//	Attribute : NominalSpdHz
	NominalSpdHzAttrib	*nominalspdhz = new NominalSpdHzAttrib();
	Tango::UserDefaultAttrProp	nominalspdhz_prop;
	nominalspdhz_prop.set_description("Velocita` nominale");
	nominalspdhz_prop.set_label("NominalSpdHz (315)");
	nominalspdhz_prop.set_unit("Hz");
	//	standard_unit	not set for NominalSpdHz
	//	display_unit	not set for NominalSpdHz
	//	format	not set for NominalSpdHz
	nominalspdhz_prop.set_max_value("999999");
	nominalspdhz_prop.set_min_value("0");
	//	max_alarm	not set for NominalSpdHz
	//	min_alarm	not set for NominalSpdHz
	//	max_warning	not set for NominalSpdHz
	//	min_warning	not set for NominalSpdHz
	//	delta_t	not set for NominalSpdHz
	//	delta_val	not set for NominalSpdHz
	
	nominalspdhz->set_default_properties(nominalspdhz_prop);
	//	Not Polled
	nominalspdhz->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(nominalspdhz);

	//	Attribute : RUTimeSVal
	RUTimeSValAttrib	*rutimesval = new RUTimeSValAttrib();
	Tango::UserDefaultAttrProp	rutimesval_prop;
	rutimesval_prop.set_description("Valore nominale tempo di accelerazione");
	rutimesval_prop.set_label("RUTimeSVal (700)");
	rutimesval_prop.set_unit("min");
	//	standard_unit	not set for RUTimeSVal
	//	display_unit	not set for RUTimeSVal
	//	format	not set for RUTimeSVal
	rutimesval_prop.set_max_value("120");
	rutimesval_prop.set_min_value("1");
	//	max_alarm	not set for RUTimeSVal
	//	min_alarm	not set for RUTimeSVal
	//	max_warning	not set for RUTimeSVal
	//	min_warning	not set for RUTimeSVal
	//	delta_t	not set for RUTimeSVal
	//	delta_val	not set for RUTimeSVal
	
	rutimesval->set_default_properties(rutimesval_prop);
	//	Not Polled
	rutimesval->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(rutimesval);

	//	Attribute : SpdSwPt1
	SpdSwPt1Attrib	*spdswpt1 = new SpdSwPt1Attrib();
	Tango::UserDefaultAttrProp	spdswpt1_prop;
	spdswpt1_prop.set_description("Punto di scatto velocita` 1");
	spdswpt1_prop.set_label("SpdSwPt1 (701)");
	spdswpt1_prop.set_unit("%");
	//	standard_unit	not set for SpdSwPt1
	//	display_unit	not set for SpdSwPt1
	//	format	not set for SpdSwPt1
	spdswpt1_prop.set_max_value("97");
	spdswpt1_prop.set_min_value("50");
	//	max_alarm	not set for SpdSwPt1
	//	min_alarm	not set for SpdSwPt1
	//	max_warning	not set for SpdSwPt1
	//	min_warning	not set for SpdSwPt1
	//	delta_t	not set for SpdSwPt1
	//	delta_val	not set for SpdSwPt1
	
	spdswpt1->set_default_properties(spdswpt1_prop);
	//	Not Polled
	spdswpt1->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(spdswpt1);

	//	Attribute : SpdSVal
	SpdSValAttrib	*spdsval = new SpdSValAttrib();
	Tango::UserDefaultAttrProp	spdsval_prop;
	spdsval_prop.set_description("Impostazione in posizionamento velocita`");
	spdsval_prop.set_label("SpdSVal (707)");
	spdsval_prop.set_unit("%");
	//	standard_unit	not set for SpdSVal
	//	display_unit	not set for SpdSVal
	//	format	not set for SpdSVal
	spdsval_prop.set_max_value("100");
	spdsval_prop.set_min_value("20");
	//	max_alarm	not set for SpdSVal
	//	min_alarm	not set for SpdSVal
	//	max_warning	not set for SpdSVal
	//	min_warning	not set for SpdSVal
	//	delta_t	not set for SpdSVal
	//	delta_val	not set for SpdSVal
	
	spdsval->set_default_properties(spdsval_prop);
	//	Not Polled
	spdsval->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(spdsval);

	//	Attribute : PwrSVal
	PwrSValAttrib	*pwrsval = new PwrSValAttrib();
	Tango::UserDefaultAttrProp	pwrsval_prop;
	pwrsval_prop.set_description("Impostazione assorbimento di potenza");
	pwrsval_prop.set_label("PwrSVal (708)");
	pwrsval_prop.set_unit("%");
	//	standard_unit	not set for PwrSVal
	//	display_unit	not set for PwrSVal
	//	format	not set for PwrSVal
	pwrsval_prop.set_max_value("100");
	pwrsval_prop.set_min_value("10");
	//	max_alarm	not set for PwrSVal
	//	min_alarm	not set for PwrSVal
	//	max_warning	not set for PwrSVal
	//	min_warning	not set for PwrSVal
	//	delta_t	not set for PwrSVal
	//	delta_val	not set for PwrSVal
	
	pwrsval->set_default_properties(pwrsval_prop);
	//	Not Polled
	pwrsval->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(pwrsval);

	//	Attribute : SwoffBKP
	SwoffBKPAttrib	*swoffbkp = new SwoffBKPAttrib();
	Tango::UserDefaultAttrProp	swoffbkp_prop;
	swoffbkp_prop.set_description("Soglia di spegnimento pompa preliminare con funzionamento a intervalli");
	swoffbkp_prop.set_label("SwoffBKP (710)");
	swoffbkp_prop.set_unit("W");
	//	standard_unit	not set for SwoffBKP
	//	display_unit	not set for SwoffBKP
	//	format	not set for SwoffBKP
	swoffbkp_prop.set_max_value("1000");
	swoffbkp_prop.set_min_value("0");
	//	max_alarm	not set for SwoffBKP
	//	min_alarm	not set for SwoffBKP
	//	max_warning	not set for SwoffBKP
	//	min_warning	not set for SwoffBKP
	//	delta_t	not set for SwoffBKP
	//	delta_val	not set for SwoffBKP
	
	swoffbkp->set_default_properties(swoffbkp_prop);
	//	Not Polled
	swoffbkp->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(swoffbkp);

	//	Attribute : SwonBKP
	SwonBKPAttrib	*swonbkp = new SwonBKPAttrib();
	Tango::UserDefaultAttrProp	swonbkp_prop;
	swonbkp_prop.set_description("Soglia di accensione pompa preliminare con funzionamento a intervalli");
	swonbkp_prop.set_label("SwonBKP (711)");
	swonbkp_prop.set_unit("W");
	//	standard_unit	not set for SwonBKP
	//	display_unit	not set for SwonBKP
	//	format	not set for SwonBKP
	swonbkp_prop.set_max_value("1000");
	swonbkp_prop.set_min_value("0");
	//	max_alarm	not set for SwonBKP
	//	min_alarm	not set for SwonBKP
	//	max_warning	not set for SwonBKP
	//	min_warning	not set for SwonBKP
	//	delta_t	not set for SwonBKP
	//	delta_val	not set for SwonBKP
	
	swonbkp->set_default_properties(swonbkp_prop);
	//	Not Polled
	swonbkp->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(swonbkp);

	//	Attribute : StdbyVal
	StdbyValAttrib	*stdbyval = new StdbyValAttrib();
	Tango::UserDefaultAttrProp	stdbyval_prop;
	stdbyval_prop.set_description("Impostazione velocita` in Standby");
	stdbyval_prop.set_label("StdbyVal (717)");
	stdbyval_prop.set_unit("%");
	//	standard_unit	not set for StdbyVal
	//	display_unit	not set for StdbyVal
	//	format	not set for StdbyVal
	stdbyval_prop.set_max_value("100");
	stdbyval_prop.set_min_value("20");
	//	max_alarm	not set for StdbyVal
	//	min_alarm	not set for StdbyVal
	//	max_warning	not set for StdbyVal
	//	min_warning	not set for StdbyVal
	//	delta_t	not set for StdbyVal
	//	delta_val	not set for StdbyVal
	
	stdbyval->set_default_properties(stdbyval_prop);
	//	Not Polled
	stdbyval->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(stdbyval);

	//	Attribute : SpdSwPt2
	SpdSwPt2Attrib	*spdswpt2 = new SpdSwPt2Attrib();
	Tango::UserDefaultAttrProp	spdswpt2_prop;
	spdswpt2_prop.set_description("Punto di scatto velocita` 2");
	spdswpt2_prop.set_label("SpdSwPt2 (719)");
	spdswpt2_prop.set_unit("%");
	//	standard_unit	not set for SpdSwPt2
	//	display_unit	not set for SpdSwPt2
	//	format	not set for SpdSwPt2
	spdswpt2_prop.set_max_value("97");
	spdswpt2_prop.set_min_value("5");
	//	max_alarm	not set for SpdSwPt2
	//	min_alarm	not set for SpdSwPt2
	//	max_warning	not set for SpdSwPt2
	//	min_warning	not set for SpdSwPt2
	//	delta_t	not set for SpdSwPt2
	//	delta_val	not set for SpdSwPt2
	
	spdswpt2->set_default_properties(spdswpt2_prop);
	//	Not Polled
	spdswpt2->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(spdswpt2);

	//	Attribute : VentSpd
	VentSpdAttrib	*ventspd = new VentSpdAttrib();
	Tango::UserDefaultAttrProp	ventspd_prop;
	ventspd_prop.set_description("Velocita` immissione aria con immissione ritardata");
	ventspd_prop.set_label("VentSpd (720)");
	ventspd_prop.set_unit("%");
	//	standard_unit	not set for VentSpd
	//	display_unit	not set for VentSpd
	//	format	not set for VentSpd
	ventspd_prop.set_max_value("98");
	ventspd_prop.set_min_value("40");
	//	max_alarm	not set for VentSpd
	//	min_alarm	not set for VentSpd
	//	max_warning	not set for VentSpd
	//	min_warning	not set for VentSpd
	//	delta_t	not set for VentSpd
	//	delta_val	not set for VentSpd
	
	ventspd->set_default_properties(ventspd_prop);
	//	Not Polled
	ventspd->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(ventspd);

	//	Attribute : VentTime
	VentTimeAttrib	*venttime = new VentTimeAttrib();
	Tango::UserDefaultAttrProp	venttime_prop;
	venttime_prop.set_description("Tempo di immissione con immissione aria ritardata");
	venttime_prop.set_label("VentTime (721)");
	venttime_prop.set_unit("s");
	//	standard_unit	not set for VentTime
	//	display_unit	not set for VentTime
	//	format	not set for VentTime
	venttime_prop.set_max_value("3600");
	venttime_prop.set_min_value("6");
	//	max_alarm	not set for VentTime
	//	min_alarm	not set for VentTime
	//	max_warning	not set for VentTime
	//	min_warning	not set for VentTime
	//	delta_t	not set for VentTime
	//	delta_val	not set for VentTime
	
	venttime->set_default_properties(venttime_prop);
	//	Not Polled
	venttime->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(venttime);

	//	Attribute : Heating
	HeatingAttrib	*heating = new HeatingAttrib();
	Tango::UserDefaultAttrProp	heating_prop;
	heating_prop.set_description("Riscaldamento");
	heating_prop.set_label("Heating (001)");
	//	unit	not set for Heating
	//	standard_unit	not set for Heating
	//	display_unit	not set for Heating
	//	format	not set for Heating
	//	max_value	not set for Heating
	//	min_value	not set for Heating
	//	max_alarm	not set for Heating
	//	min_alarm	not set for Heating
	//	max_warning	not set for Heating
	//	min_warning	not set for Heating
	//	delta_t	not set for Heating
	//	delta_val	not set for Heating
	
	heating->set_default_properties(heating_prop);
	//	Not Polled
	heating->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(heating);

	//	Attribute : Standby
	StandbyAttrib	*standby = new StandbyAttrib();
	Tango::UserDefaultAttrProp	standby_prop;
	standby_prop.set_description("Standby");
	standby_prop.set_label("Standby (002)");
	//	unit	not set for Standby
	//	standard_unit	not set for Standby
	//	display_unit	not set for Standby
	//	format	not set for Standby
	//	max_value	not set for Standby
	//	min_value	not set for Standby
	//	max_alarm	not set for Standby
	//	min_alarm	not set for Standby
	//	max_warning	not set for Standby
	//	min_warning	not set for Standby
	//	delta_t	not set for Standby
	//	delta_val	not set for Standby
	
	standby->set_default_properties(standby_prop);
	//	Not Polled
	standby->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(standby);

	//	Attribute : RUTimeCtrl
	RUTimeCtrlAttrib	*rutimectrl = new RUTimeCtrlAttrib();
	Tango::UserDefaultAttrProp	rutimectrl_prop;
	rutimectrl_prop.set_description("Monitoraggio tempo di accelerazione");
	rutimectrl_prop.set_label("RUTimeCtrl (004)");
	//	unit	not set for RUTimeCtrl
	//	standard_unit	not set for RUTimeCtrl
	//	display_unit	not set for RUTimeCtrl
	//	format	not set for RUTimeCtrl
	//	max_value	not set for RUTimeCtrl
	//	min_value	not set for RUTimeCtrl
	//	max_alarm	not set for RUTimeCtrl
	//	min_alarm	not set for RUTimeCtrl
	//	max_warning	not set for RUTimeCtrl
	//	min_warning	not set for RUTimeCtrl
	//	delta_t	not set for RUTimeCtrl
	//	delta_val	not set for RUTimeCtrl
	
	rutimectrl->set_default_properties(rutimectrl_prop);
	//	Not Polled
	rutimectrl->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(rutimectrl);

	//	Attribute : ErrorAckn
	ErrorAcknAttrib	*errorackn = new ErrorAcknAttrib();
	Tango::UserDefaultAttrProp	errorackn_prop;
	errorackn_prop.set_description("Conferma guasti");
	errorackn_prop.set_label("ErrorAckn (009)");
	//	unit	not set for ErrorAckn
	//	standard_unit	not set for ErrorAckn
	//	display_unit	not set for ErrorAckn
	//	format	not set for ErrorAckn
	//	max_value	not set for ErrorAckn
	//	min_value	not set for ErrorAckn
	//	max_alarm	not set for ErrorAckn
	//	min_alarm	not set for ErrorAckn
	//	max_warning	not set for ErrorAckn
	//	min_warning	not set for ErrorAckn
	//	delta_t	not set for ErrorAckn
	//	delta_val	not set for ErrorAckn
	
	errorackn->set_default_properties(errorackn_prop);
	//	Not Polled
	errorackn->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(errorackn);

	//	Attribute : PumpgStatn
	PumpgStatnAttrib	*pumpgstatn = new PumpgStatnAttrib();
	Tango::UserDefaultAttrProp	pumpgstatn_prop;
	pumpgstatn_prop.set_description("Stato pompa");
	pumpgstatn_prop.set_label("PumpgStatn (010)");
	//	unit	not set for PumpgStatn
	//	standard_unit	not set for PumpgStatn
	//	display_unit	not set for PumpgStatn
	//	format	not set for PumpgStatn
	//	max_value	not set for PumpgStatn
	//	min_value	not set for PumpgStatn
	//	max_alarm	not set for PumpgStatn
	//	min_alarm	not set for PumpgStatn
	//	max_warning	not set for PumpgStatn
	//	min_warning	not set for PumpgStatn
	//	delta_t	not set for PumpgStatn
	//	delta_val	not set for PumpgStatn
	
	pumpgstatn->set_default_properties(pumpgstatn_prop);
	//	Not Polled
	pumpgstatn->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(pumpgstatn);

	//	Attribute : EnableVent
	EnableVentAttrib	*enablevent = new EnableVentAttrib();
	Tango::UserDefaultAttrProp	enablevent_prop;
	enablevent_prop.set_description("Rilascio immisione aria");
	enablevent_prop.set_label("EnableVent (012)");
	//	unit	not set for EnableVent
	//	standard_unit	not set for EnableVent
	//	display_unit	not set for EnableVent
	//	format	not set for EnableVent
	//	max_value	not set for EnableVent
	//	min_value	not set for EnableVent
	//	max_alarm	not set for EnableVent
	//	min_alarm	not set for EnableVent
	//	max_warning	not set for EnableVent
	//	min_warning	not set for EnableVent
	//	delta_t	not set for EnableVent
	//	delta_val	not set for EnableVent
	
	enablevent->set_default_properties(enablevent_prop);
	//	Not Polled
	enablevent->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(enablevent);

	//	Attribute : MotorPump
	MotorPumpAttrib	*motorpump = new MotorPumpAttrib();
	Tango::UserDefaultAttrProp	motorpump_prop;
	motorpump_prop.set_description("Motore pompa");
	motorpump_prop.set_label("MotorPump (023)");
	//	unit	not set for MotorPump
	//	standard_unit	not set for MotorPump
	//	display_unit	not set for MotorPump
	//	format	not set for MotorPump
	//	max_value	not set for MotorPump
	//	min_value	not set for MotorPump
	//	max_alarm	not set for MotorPump
	//	min_alarm	not set for MotorPump
	//	max_warning	not set for MotorPump
	//	min_warning	not set for MotorPump
	//	delta_t	not set for MotorPump
	//	delta_val	not set for MotorPump
	
	motorpump->set_default_properties(motorpump_prop);
	//	Not Polled
	motorpump->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(motorpump);

	//	Attribute : OpModeBKP
	OpModeBKPAttrib	*opmodebkp = new OpModeBKPAttrib();
	Tango::UserDefaultAttrProp	opmodebkp_prop;
	opmodebkp_prop.set_description("Tipo di azionamento pompa preliminare");
	opmodebkp_prop.set_label("OpModeBKP (025)");
	//	unit	not set for OpModeBKP
	//	standard_unit	not set for OpModeBKP
	//	display_unit	not set for OpModeBKP
	//	format	not set for OpModeBKP
	opmodebkp_prop.set_max_value("2");
	opmodebkp_prop.set_min_value("0");
	//	max_alarm	not set for OpModeBKP
	//	min_alarm	not set for OpModeBKP
	//	max_warning	not set for OpModeBKP
	//	min_warning	not set for OpModeBKP
	//	delta_t	not set for OpModeBKP
	//	delta_val	not set for OpModeBKP
	
	opmodebkp->set_default_properties(opmodebkp_prop);
	//	Not Polled
	opmodebkp->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(opmodebkp);

	//	Attribute : SpdSetMode
	SpdSetModeAttrib	*spdsetmode = new SpdSetModeAttrib();
	Tango::UserDefaultAttrProp	spdsetmode_prop;
	spdsetmode_prop.set_description("Modalita di posizionamento di velocita`");
	spdsetmode_prop.set_label("SpdSetMode (026)");
	//	unit	not set for SpdSetMode
	//	standard_unit	not set for SpdSetMode
	//	display_unit	not set for SpdSetMode
	//	format	not set for SpdSetMode
	spdsetmode_prop.set_max_value("1");
	spdsetmode_prop.set_min_value("0");
	//	max_alarm	not set for SpdSetMode
	//	min_alarm	not set for SpdSetMode
	//	max_warning	not set for SpdSetMode
	//	min_warning	not set for SpdSetMode
	//	delta_t	not set for SpdSetMode
	//	delta_val	not set for SpdSetMode
	
	spdsetmode->set_default_properties(spdsetmode_prop);
	//	Not Polled
	spdsetmode->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(spdsetmode);

	//	Attribute : GasMode
	GasModeAttrib	*gasmode = new GasModeAttrib();
	Tango::UserDefaultAttrProp	gasmode_prop;
	gasmode_prop.set_description("Modalita gas");
	gasmode_prop.set_label("GasMode (027)");
	//	unit	not set for GasMode
	//	standard_unit	not set for GasMode
	//	display_unit	not set for GasMode
	//	format	not set for GasMode
	gasmode_prop.set_max_value("2");
	gasmode_prop.set_min_value("0");
	//	max_alarm	not set for GasMode
	//	min_alarm	not set for GasMode
	//	max_warning	not set for GasMode
	//	min_warning	not set for GasMode
	//	delta_t	not set for GasMode
	//	delta_val	not set for GasMode
	
	gasmode->set_default_properties(gasmode_prop);
	//	Not Polled
	gasmode->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(gasmode);

	//	Attribute : VentMode
	VentModeAttrib	*ventmode = new VentModeAttrib();
	Tango::UserDefaultAttrProp	ventmode_prop;
	ventmode_prop.set_description("Modalita` immissione aria");
	ventmode_prop.set_label("VentMode (030)");
	//	unit	not set for VentMode
	//	standard_unit	not set for VentMode
	//	display_unit	not set for VentMode
	//	format	not set for VentMode
	ventmode_prop.set_max_value("2");
	ventmode_prop.set_min_value("0");
	//	max_alarm	not set for VentMode
	//	min_alarm	not set for VentMode
	//	max_warning	not set for VentMode
	//	min_warning	not set for VentMode
	//	delta_t	not set for VentMode
	//	delta_val	not set for VentMode
	
	ventmode->set_default_properties(ventmode_prop);
	//	Not Polled
	ventmode->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(ventmode);

	//	Attribute : CfgAO1
	CfgAO1Attrib	*cfgao1 = new CfgAO1Attrib();
	Tango::UserDefaultAttrProp	cfgao1_prop;
	cfgao1_prop.set_description("Configurazione uscita AO1");
	cfgao1_prop.set_label("CfgAO1 (055)");
	//	unit	not set for CfgAO1
	//	standard_unit	not set for CfgAO1
	//	display_unit	not set for CfgAO1
	//	format	not set for CfgAO1
	cfgao1_prop.set_max_value("4");
	cfgao1_prop.set_min_value("0");
	//	max_alarm	not set for CfgAO1
	//	min_alarm	not set for CfgAO1
	//	max_warning	not set for CfgAO1
	//	min_warning	not set for CfgAO1
	//	delta_t	not set for CfgAO1
	//	delta_val	not set for CfgAO1
	
	cfgao1->set_default_properties(cfgao1_prop);
	//	Not Polled
	cfgao1->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(cfgao1);

	//	Attribute : ErrHist
	ErrHistAttrib	*errhist = new ErrHistAttrib();
	Tango::UserDefaultAttrProp	errhist_prop;
	errhist_prop.set_description("Cronologia codici errori");
	errhist_prop.set_label("ErrHist (360-369)");
	//	unit	not set for ErrHist
	//	standard_unit	not set for ErrHist
	//	display_unit	not set for ErrHist
	//	format	not set for ErrHist
	//	max_value	not set for ErrHist
	//	min_value	not set for ErrHist
	//	max_alarm	not set for ErrHist
	//	min_alarm	not set for ErrHist
	//	max_warning	not set for ErrHist
	//	min_warning	not set for ErrHist
	//	delta_t	not set for ErrHist
	//	delta_val	not set for ErrHist
	
	errhist->set_default_properties(errhist_prop);
	//	Not Polled
	errhist->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(errhist);


	//	Create a list of static attributes
	create_static_attribute_list(get_class_attr()->get_attr_list());
	/*----- PROTECTED REGION ID(HipaceClass::attribute_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::attribute_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : HipaceClass::pipe_factory()
 *	Description : Create the pipe object(s)
 *                and store them in the pipe list
 */
//--------------------------------------------------------
void HipaceClass::pipe_factory()
{
	/*----- PROTECTED REGION ID(HipaceClass::pipe_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::pipe_factory_before
	/*----- PROTECTED REGION ID(HipaceClass::pipe_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::pipe_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : HipaceClass::command_factory()
 *	Description : Create the command object(s)
 *                and store them in the command list
 */
//--------------------------------------------------------
void HipaceClass::command_factory()
{
	/*----- PROTECTED REGION ID(HipaceClass::command_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::command_factory_before
	//	Call command_factory for inherited class
	Serial2Client_ns::Serial2ClientClass::command_factory();



	/*----- PROTECTED REGION ID(HipaceClass::command_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::command_factory_after
}

//===================================================================
//	Dynamic attributes related methods
//===================================================================

//--------------------------------------------------------
/**
 * method : 		HipaceClass::create_static_attribute_list
 * description : 	Create the a list of static attributes
 *
 * @param	att_list	the ceated attribute list
 */
//--------------------------------------------------------
void HipaceClass::create_static_attribute_list(vector<Tango::Attr *> &att_list)
{
	for (unsigned long i=0 ; i<att_list.size() ; i++)
	{
		string att_name(att_list[i]->get_name());
		transform(att_name.begin(), att_name.end(), att_name.begin(), ::tolower);
		defaultAttList.push_back(att_name);
	}

	cout2 << defaultAttList.size() << " attributes in default list" << endl;

	/*----- PROTECTED REGION ID(HipaceClass::create_static_att_list) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::create_static_att_list
}


//--------------------------------------------------------
/**
 * method : 		HipaceClass::erase_dynamic_attributes
 * description : 	delete the dynamic attributes if any.
 *
 * @param	devlist_ptr	the device list pointer
 * @param	list of all attributes
 */
//--------------------------------------------------------
void HipaceClass::erase_dynamic_attributes(const Tango::DevVarStringArray *devlist_ptr, vector<Tango::Attr *> &att_list)
{
	Tango::Util *tg = Tango::Util::instance();

	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		Tango::DeviceImpl *dev_impl = tg->get_device_by_name(((string)(*devlist_ptr)[i]).c_str());
		Hipace *dev = static_cast<Hipace *> (dev_impl);

		vector<Tango::Attribute *> &dev_att_list = dev->get_device_attr()->get_attribute_list();
		vector<Tango::Attribute *>::iterator ite_att;
		for (ite_att=dev_att_list.begin() ; ite_att != dev_att_list.end() ; ++ite_att)
		{
			string att_name((*ite_att)->get_name_lower());
			if ((att_name == "state") || (att_name == "status"))
				continue;
			vector<string>::iterator ite_str = find(defaultAttList.begin(), defaultAttList.end(), att_name);
			if (ite_str == defaultAttList.end())
			{
				cout2 << att_name << " is a UNWANTED dynamic attribute for device " << (*devlist_ptr)[i] << endl;
				Tango::Attribute &att = dev->get_device_attr()->get_attr_by_name(att_name.c_str());
				dev->remove_attribute(att_list[att.get_attr_idx()], true, false);
				--ite_att;
			}
		}
	}
	/*----- PROTECTED REGION ID(HipaceClass::erase_dynamic_attributes) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	HipaceClass::erase_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method      : HipaceClass::get_attr_by_name()
 *	Description : returns Tango::Attr * object found by name
 */
//--------------------------------------------------------
Tango::Attr *HipaceClass::get_attr_object_by_name(vector<Tango::Attr *> &att_list, string attname)
{
	vector<Tango::Attr *>::iterator it;
	for (it=att_list.begin() ; it<att_list.end() ; ++it)
		if ((*it)->get_name()==attname)
			return (*it);
	//	Attr does not exist
	return NULL;
}


/*----- PROTECTED REGION ID(HipaceClass::Additional Methods) ENABLED START -----*/

/*----- PROTECTED REGION END -----*/	//	HipaceClass::Additional Methods
} //	namespace
